#!/usr/local/bin/bltwish

package require BLT

set Graph(dataName) close.data
set name $argv

################################################################
# proc normalize {val}--
#    Convert fractions to decimals and remove any commas
# Arguments
#   val		A numeric value
# 
# Results
#   Returns a legal floating point or integer value.
# 
proc normalize {val} {

    # Only do fraction to decimal conversion if a fraction exists.

    if {[regexp {([0-9]* +)*([0-9]*)/([0-9]*)} $val m whole num denom]} {
        set val [expr $whole + ($num / $denom.0)]
    }
    
    # Delete any commas that might be in the value
    regsub -all "," $val "" val

    return $val
}


################################################################
# proc readData {infl}--
#    Reads data from a stock data file
# Arguments
#   infl	A channel to the data file
# 
# Results
#   Creates lists of values in the Data global associative array,
#   sorted by StockSymbol.DataType
# 
proc readData {infl} {
    global Data
    
    while {[set len [gets $infl line]] >= 0} {
        set id [lindex $line 1]
        foreach n {date id price change pct dt o high low vol} v $line {
            lappend Data($id.$n) [normalize $v]
        }
    }
}

################################################################
# proc fmt {graph sec}--
#    Formats a time-since-epoch time into MM/DD
# Arguments
#   graph	The name of the graph which includes this tic mark.
#   sec		Seconds since the Epoch.
# Results
#   Returns the appropriate MM/DD value.
# 
proc fmt {graph sec} {
   return [clock format $sec -format {%m/%d}]
}

################################################################
# proc makeGraph {parent name}--
#    Makes a stock graph.
# Arguments
#   parent	A parent frame to hold this graph
#   name	The stock symbol to use as a key to access the data
#               to display in this graph.
# Results
#   Creates a new graph widget, and returns the name of that widget.
# 
proc makeGraph {parent name} {
  global Data


  if {![winfo exists $parent.g_$name]} {
      # Create the graph
      ::blt::graph $parent.g_$name -title "Stock Data: $name" -width 600 -height 400 
      
      # Format the x-axis tick labels as Month/Day
      $parent.g_$name axis configure x -command fmt
      
      # Create a line showing the stock price when the robot ran.
      $parent.g_$name element create "$name Price" -xdata $Data($name.date) \
          -ydata $Data($name.price) -symbol none 
      
      # Generate a bar chart for volume, and display the second Y axis
      #   that the bar chart references

      $parent.g_$name bar create "$name Volume" -xdata $Data($name.date) \
           -ydata $Data($name.vol) -mapy y2 -fg green -bg green \
	   -barwidth 2000
  
      $parent.g_$name axis configure y2 -hide 0
      

      # Do an update to force the graph to run through the data and
      #   canculate the min and max values.
      
      update idle;
      set max [$parent.g_$name axis cget y -max]
      set min [$parent.g_$name axis cget y -min]

      # Create vertical high/low lines at the vertices, and find max & min.
      foreach d $Data($name.date) h $Data($name.high) l $Data($name.low) {

          $parent.g_$name marker create line -coords [list $d $h $d $l] \
	      -outline red

	  if {$l < $min} {set min $l}
	  if {$h > $max} {set max $h}
      }

      # Now expand the Y axis to the real min/max range.

      $parent.g_$name axis configure y -max $max
      $parent.g_$name axis configure y -min $min
   }

  return $parent.g_$name
}

set infl [open $Graph(dataName) r]
readData $infl
close $infl

set w [frame .graphs]

pack $w -side bottom

pack [makeGraph $w $name]

